;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3485df92a8c61dcfa39068c960365084 . "./code.cg2js/cg-acache.fasl")
(#xc2af7e9f3cb1ae64ac421f1256433cc8 . "./code.cg2js/cg-base.fasl")
(#x790a7d49346906a66295681ef8b41bd2 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xc8af5903c97a44f493cd8a136be40313 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4643289bca895b80a5ecabd52b69811e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x676f4f08b6dcde67af335a2f244ee506 . "./code.cg2js/cg-button.fasl")
(#x1828e73e52ce7dfb876864fe103e0f68 . "./code.cg2js/cg-calendar.fasl")
(#x5a75fa066a0ccae03e75ce17c41647df . "./code.cg2js/cg-caret.fasl")
(#xc357a3c7b79bbf71b2587febc3966fe1 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x511139749f0ec7b7e1faf7c28a599ba8 . "./code.cg2js/cg-chart-widget.fasl")
(#x1893bc09b1af102b54d17a4649e70c83 . "./code.cg2js/cg-check-box.fasl")
(#x79b0a58e62ce1ed1e05731e4c3766107 . "./code.cg2js/cg-choice-list.fasl")
(#x04245b716ad6e57ef042dddfd63239aa . "./code.cg2js/cg-choose-printer.fasl")
(#xa589c9151163f2689977566c39df647b . "./code.cg2js/cg-class-grid.fasl")
(#x1aa3ef26c8516dfe29ccb506fe818788 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x36d8a384cd0e00884beeb99382a03922 . "./code.cg2js/cg-class-support.fasl")
(#x85afd7b4fbc2b30a4a77ccf283166bc0 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x6cb7acc5921efc8e46084a9a19237537 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x9ea970ad7c1f82b5d5bc5faed604c5fa . "./code.cg2js/cg-clipboard.fasl")
(#x9b05ac4dd4df1a407fd0cd2aef195243 . "./code.cg2js/cg-color-dialog.fasl")
(#xb5848010cde43bf9cd70ff9e5eb95f43 . "./code.cg2js/cg-combo-box.fasl")
(#x44b4a715396476d7089c4437927339ad . "./code.cg2js/cg-common-control.fasl")
(#x75c2a65f7538f29155ed9fd8935ac184 . "./code.cg2js/cg-comtab.fasl")
(#x79c09f84d72934b104c761aec97240f4 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x92b0a788d94238312fe6e6ab4a4a6825 . "./code.cg2js/cg-curve.fasl")
(#x3faf3ebbb691cd76f74d709f33a14856 . "./code.cg2js/cg-dde-utils.fasl")
(#x052e5ad94757d18b0952d038e2c6e76e . "./code.cg2js/cg-dev.fasl")
(#xb26362d82c95e558219d1d86e6890cfa . "./code.cg2js/cg-dialog-item.fasl")
(#xc8cabe2066e97889fe31f09759aadaa6 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe4b2853230f720dbce75a8636779a63f . "./code.cg2js/cg-directory-dialog.fasl")
(#xf05d8930d05c273e58f2cc19e7646c7d . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x78316f166f5ab7e9c72c3d58dbaf75a9 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2a726d9c78d42339ba119e26ff66875d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x9ea7bfcd0cb17c7afe936355e4c3e73c . "./code.cg2js/cg-drawable.fasl")
(#x4f7ab2a408b19183a066d67f1d8e62a6 . "./code.cg2js/cg-dropping-outline.fasl")
(#x0436490fd0065c5ae0b39dbdfb7ef124 . "./code.cg2js/cg-edit-in-place.fasl")
(#xb1c59b9b314ab28ebd976d856b9d2c4b . "./code.cg2js/cg-editable-text.fasl")
(#xfe600c8ce8ccd8fbaa965d542da52d2f . "./code.cg2js/cg-file-dialog.fasl")
(#x7a4df6e952504ce4a11d6d5f580fc4b1 . "./code.cg2js/cg-fill-texture.fasl")
(#xb9d59888568d9b70e080791880759802 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xfe5eaea3b57f62a94567d0050bcd1ecd . "./code.cg2js/cg-font-dialog.fasl")
(#x6fbbc775b7ff53979ed76d8383b4b1f2 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x19682e6b431059989760be72f59c6611 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7f5f43d0ff5a642f113f7c36dcc6fc46 . "./code.cg2js/cg-get-position.fasl")
(#x8143859fa92ce096ffc80f76c6289b92 . "./code.cg2js/cg-graphics-context.fasl")
(#xe9182bfca1074b2a780173e95869dac7 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x01956e901886c75cfdac7f9a6510ee59 . "./code.cg2js/cg-grid-widget.fasl")
(#xfe339d0af988b7a278189e3c5f75b92a . "./code.cg2js/cg-group-box.fasl")
(#x8527b67e7eed3f21dda8d6b6027b0899 . "./code.cg2js/cg-header-control.fasl")
(#x6c82958c7ce13bf9c461022d6b10e195 . "./code.cg2js/cg-hotspot.fasl")
(#x642485809661b8782c438222073026be . "./code.cg2js/cg-html-dialog.fasl")
(#x596cad244a9fbae828347be1b3bf985f . "./code.cg2js/cg-html-widget.fasl")
(#x233c96d659973e2e07b8436dd9eb321f . "./code.cg2js/cg-icon-pixmap.fasl")
(#xea1618495ec3b4ffe2d76a83672cbf86 . "./code.cg2js/cg-icon.fasl")
(#x65a0bbf0e5d3ffc168db1f9bfbc86147 . "./code.cg2js/cg-ie.fasl")
(#xf8408aeb8d7828b9c82465a49719308b . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc4197e964030144a7eabb830b176526d . "./code.cg2js/cg-intersect.fasl")
(#x2fadf7b5204dae74ae1743e1e0531533 . "./code.cg2js/cg-item-list.fasl")
(#xafeb22d74056b9ebfbabe04047098c5c . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4e3b753727613761208ab3e9e45b573b . "./code.cg2js/cg-lamp.fasl")
(#x86188f59309a87e8bda8f902d41cc3ae . "./code.cg2js/cg-layout.fasl")
(#xe051af585dee261b244fd581627d66d3 . "./code.cg2js/cg-lettered-menu.fasl")
(#x0cc1d5a7bdc6b00e92c74209649a769c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x36a1b4e32d6518f9c564a9ac592e933b . "./code.cg2js/cg-lisp-text.fasl")
(#x41d98c7368dfc1b50550c69eab462bcf . "./code.cg2js/cg-lisp-widget.fasl")
(#xaaf04cc6efea5072b66610d7f5ec35e9 . "./code.cg2js/cg-list-view.fasl")
(#x6b7b947a00cf41ddb74d2dde320a5611 . "./code.cg2js/cg-mci.fasl")
(#xc3fedc480435b016676de0b942e5f8e5 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xb0ad27489a91375cc7ecd7802c5b1b8a . "./code.cg2js/cg-menu.fasl")
(#xc587ff35919f4e07fac85b552a79eef6 . "./code.cg2js/cg-message-dialog.fasl")
(#xa5cf92e4a2a212b4967a36b984d078a5 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x50fd020c5a167778a78873961525471b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xf4f3f570a3545b58592eaceb981c5123 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x40bc6d0b4bf658d7dc700296c09bfcea . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x1979f892afd8afae847f6ab621ab38cc . "./code.cg2js/cg-multi-picture-button.fasl")
(#x66a4ff24c0cfc537f832fc1b1505a858 . "./code.cg2js/cg-nodes.fasl")
(#x007ac459a2bf62b2ac3b2e95b42e12bb . "./code.cg2js/cg-object-editor-layout.fasl")
(#x04d48ee37856f5ea4fdfcf05663c9699 . "./code.cg2js/cg-object-editor.fasl")
(#x0a019b77d4e29304a4b943b50dd34977 . "./code.cg2js/cg-ocx.fasl")
(#x9e3196cf1333c87b8a7c329e756cd398 . "./code.cg2js/cg-os-widget.fasl")
(#xd2f6763fc9856b5aa7eceb2d7c3797bb . "./code.cg2js/cg-os-window.fasl")
(#x4bc5695f16463d6b6b1fef24fc7cced1 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xcb1e675ce29ff9b6c8cd1e4d60ff0d67 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf08752d36f328e1f58d55756772980c3 . "./code.cg2js/cg-outline.fasl")
(#xd8674b541fa4a24204f70797582d130b . "./code.cg2js/cg-palette.fasl")
(#xceecf9ef0721efd9e2fb0d135af2bc8d . "./code.cg2js/cg-paren-matching.fasl")
(#xbb41f6c0c8d673db1e76fad0461d4543 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6197034e928ce724d8e1a764d29e31c7 . "./code.cg2js/cg-picture-widget.fasl")
(#x5eee385d4f3ee843d1a2e083bac6d743 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x6994bad4ce61a05f6656b3287511866d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x974ff1b76bab79807dd2c6138b4209fa . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8bec4b905993f984771cb36b55d5198e . "./code.cg2js/cg-pixmap-widget.fasl")
(#x64a43d0330a103d59122b54552584561 . "./code.cg2js/cg-pixmap.fasl")
(#xbe1c7fa53cb26e976186c233159ff9fb . "./code.cg2js/cg-player.fasl")
(#x4e360c72cae878c16965c34a7bc426f6 . "./code.cg2js/cg-plot-widget.fasl")
(#xbba64648de252c0e61585a43ca496d73 . "./code.cg2js/cg-printing.fasl")
(#xa6f6e23515dd6c140ad35fb55d7f2804 . "./code.cg2js/cg-progress-indicator.fasl")
(#x58d97f519a45a2f23347a9f5e9fe0808 . "./code.cg2js/cg-project-window.fasl")
(#x677c7a25bd6f5231fce4a4fa4974f232 . "./code.cg2js/cg-property.fasl")
(#x331adba9d3ec1b7bfd7c2623b5480bf1 . "./code.cg2js/cg-radio-button.fasl")
(#xdf38a737a0788d5db4f8b595e125670c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe8d342f51d14d2bbe8bf4fcdd0d91382 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x270a16efca9a96993725c04598cd48b3 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x19f46fcc9e5be0ad4ffb319cf9a77276 . "./code.cg2js/cg-rich-edit.fasl")
(#xf535649b0c77d743a20e8a41ea71992c . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe02a0aeb534adc8df49a46ea1f487995 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3ce5110a06ab43af7503417e781332f0 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xfa4793351b68069197d81ff5c872dd34 . "./code.cg2js/cg-scroll-bar.fasl")
(#x2672fe8e1842d33f5228e0fde3d630ba . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xeb0b937d1089f9aa8fb5d33b201d7bdf . "./code.cg2js/cg-selected-object.fasl")
(#xb35c714b80394e8b4942c49ee60ee1c1 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x6375d87635e2da624cb5d0a0ac0bccdb . "./code.cg2js/cg-split-bar.fasl")
(#xc4579f209edbed4f30c466874df5c39d . "./code.cg2js/cg-static-text.fasl")
(#x1fdbe8321f662318bf8571f05bcc1597 . "./code.cg2js/cg-status-bar.fasl")
(#xd6d82f69306e66b94ee3b48901503b39 . "./code.cg2js/cg-string-dialog.fasl")
(#x3e4c353f5f7b1393faad758dce845e81 . "./code.cg2js/cg-tab-control.fasl")
(#xf3f21a9482c0d4e4da869343806d424c . "./code.cg2js/cg-template-string.fasl")
(#xe514eaafda48ed39434072af8e04cc8a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4cb1d16aea55b4f7b6bcd5277e0469ff . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4eda35248a45c9f1cbc8915005ab63bc . "./code.cg2js/cg-text-edit-pane.fasl")
(#x35448538e9397076633d53439f1a3c0f . "./code.cg2js/cg-text-or-combo.fasl")
(#x15c5de150bcd0ccff4387f0ac0b83020 . "./code.cg2js/cg-text-widget.fasl")
(#x8a097195cd4044762d894f81d8f6029a . "./code.cg2js/cg-timer.fasl")
(#x72273937d542e4a98df5ea70ec41b09b . "./code.cg2js/cg-toggling-widget.fasl")
(#xbfc113e32c61e0de4bec752e6bea01e8 . "./code.cg2js/cg-toolbar.fasl")
(#xcfd628a4fa0d41e28ec182f871c7b5dd . "./code.cg2js/cg-tooltip.fasl")
(#x751c6edad14c1cfd257fd1684ad724a9 . "./code.cg2js/cg-touch.fasl")
(#xa3e6cae223fff40394121b6933bbb2b9 . "./code.cg2js/cg-trackbar.fasl")
(#x1ed834c356012199c1ab9325bcd0bedb . "./code.cg2js/cg-tray.fasl")
(#x2ddbfc5342cd4aa2a3353cc8446e7c9a . "./code.cg2js/cg-up-down-control.fasl")
(#xeb39412063c1fcfef3f26cfefa86ba81 . "./code.cg2js/cg-utility-dialog.fasl")
(#xc74defb8e660fe9756702837ada43470 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x55bdf9b560a78907e43af6f1ef2170a7 . "./code.cg2js/cg-web-browser.fasl")
(#xcabd0864b866b77aaa8525980a112bc6 . "./code.cg2js/cg-wrap-string.fasl")
(#x9f2e3bc0d12a37a43ff9b40b98127cb4 . "./code.cg2js/cg-yes-no-list.fasl")
(#xbfd058740c59c9688745b7c3fde0525a . "./code.cg2js/cg.fasl")
(#xa80d0da3a263d7577acb9b619ead199b . "./code.cg2js/cg-yes-no-string.fasl")
(#xea1b1bbe8f9f4b8b91efe44c33cbadd9 . "./code.idejs/ide-acache.fasl")
(#x5babd0e527da44a8eb20215a98b2e9fe . "./code.idejs/ide-apropos.fasl")
(#x9a263c0dc1e15116104738d263f44274 . "./code.idejs/ide-base.fasl")
(#x448ca3ee9ebed3bd63f6047b01b0c1f3 . "./code.idejs/ide-class-browser.fasl")
(#x303b5dab50bb877f5c62e51b800bc035 . "./code.idejs/ide-class-grapher.fasl")
(#xfd278f56cfcd73ebd67115c3ab4637ab . "./code.idejs/ide-clipboard.fasl")
(#x2d5ed05cb81ce195f14dae661cf4c21c . "./code.idejs/ide-dev.fasl")
(#x591e9a6624b79644a599ca06fcb9fd5c . "./code.idejs/ide-editor.fasl")
(#x0654a3f7c6095f61e8ca61024b7042b5 . "./code.idejs/ide-find-definitions.fasl")
(#xa26e9cef85f969acb9b06c5005442c58 . "./code.idejs/ide-find-in-files.fasl")
(#x06807745ac6259296477f13ed0611245 . "./code.idejs/ide-form.fasl")
(#x232cca89563ce1e8e5ba21ed3bcb0aa0 . "./code.idejs/ide-grapher.fasl")
(#x3107a4678dc6158ba45a2927c026f105 . "./code.idejs/ide-inspector.fasl")
(#x1e9d23921273dc6d072399e1be0377d0 . "./code.idejs/ide-intro.fasl")
(#x94ceeb100c213b697640cd5f403dfd03 . "./code.idejs/ide-keyshort.fasl")
(#xc08c7ba0371e0984b4d290066de2e80c . "./code.idejs/ide-navigator.fasl")
(#x0198af58c5ff4e399c536d4745e0154c . "./code.idejs/ide-package-list.fasl")
(#x8ce4ea01b9775c79d67e01a891e534b8 . "./code.idejs/ide-patch-dialog.fasl")
(#xea12cef5e65f227996edf8c6f0825e65 . "./code.idejs/ide-process-browser.fasl")
(#x15631c29b753083ab12e18d5154993f4 . "./code.idejs/ide-project.fasl")
(#x70c5712c4573ec09665b4029822cc78a . "./code.idejs/ide-trace.fasl")
(#xeb6b7b9acc10dfe2cd933d06bc30a393 . "./code.idejs/ide-window-list.fasl")
(#x036a314061daeb8c50766dd665542f0f . "./code.idejs/ide.fasl")
(#xde049c9404b4ba293f0cbc6a4817dc96 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
